/**
 * Copyright © 2020 Johnson & Johnson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* eslint-disable jsx-a11y/anchor-is-valid */
import React from 'react';
import { withNode, useNode } from '@bodiless/core';
import { designable, addProps, Fragment, withDesign, replaceWith, withoutProps, flowIf, } from '@bodiless/fclasses';
import { observer } from 'mobx-react';
import flowRight from 'lodash/flowRight';
import { asStylableBreadcrumbs } from './Breadcrumb.token';
import withBreadcrumbItemsFromStore from './withBreadcrumbItemsFromStore';
import withBreadcrumbsSD from './withBreadcrumbsStructuredData';
import MenuTitle from '../Menu/MenuTitles';
var ItemNodeProvider = withNode(Fragment);
var useIsLink = function (linkNodeKey) {
    var node = useNode().node;
    var linkHref = node.child(linkNodeKey);
    return Boolean(linkHref.data.href);
};
var BreadcrumbsClean$ = function (props) {
    var _a = props.hasStartingTrail, hasStartingTrail = _a === void 0 ? false : _a, _b = props.items, items = _b === void 0 ? [] : _b, _c = props.hasFinalTrail, hasFinalTrail = _c === void 0 ? false : _c, _d = props.renderLastItemWithoutLink, renderLastItemWithoutLink = _d === void 0 ? false : _d, C = props.components;
    var hasStartingTrail$ = typeof hasStartingTrail === 'function' ? hasStartingTrail() : hasStartingTrail;
    var hasFinalTrail$ = typeof hasFinalTrail === 'function' ? hasFinalTrail() : hasFinalTrail;
    var renderLastItemWithoutLink$ = typeof renderLastItemWithoutLink === 'function'
        ? renderLastItemWithoutLink()
        : renderLastItemWithoutLink;
    var items$ = items.map(function (item, index) {
        var isLastItem = index === (items.length - 1);
        var position = item.position, isCurrentPage = item.isCurrentPage;
        // increment position by 1 if there is starting trail item
        var position$ = hasStartingTrail$ ? position + 1 : position;
        if (isLastItem && renderLastItemWithoutLink$) {
            var TitleWithNoLink = withDesign({
                Link: replaceWith(Fragment),
            })(C.Title);
            return (React.createElement(React.Fragment, { key: item.uuid },
                React.createElement(C.Item, { position: position$, isCurrentPage: isCurrentPage },
                    React.createElement(ItemNodeProvider, { nodeKey: item.nodeKey, nodeCollection: item.nodeCollection },
                        React.createElement(TitleWithNoLink, { isCurrentPage: isCurrentPage })))));
        }
        // Replace Link with Fragment if no link data found.
        var Title = withDesign({
            Link: flowIf(function () { return !(useIsLink('link')); })(replaceWith(Fragment)),
        })(C.Title);
        return (React.createElement(React.Fragment, { key: item.uuid },
            React.createElement(C.Item, { position: position$, isCurrentPage: isCurrentPage },
                React.createElement(ItemNodeProvider, { nodeKey: item.nodeKey, nodeCollection: item.nodeCollection },
                    React.createElement(Title, { isCurrentPage: isCurrentPage }))),
            !isLastItem && React.createElement(C.Separator, { key: "separator".concat(item.uuid) })));
    });
    var finalTrailItemPosition = (hasStartingTrail$ ? 1 : 0) + items$.length + 1;
    return (React.createElement(C.NavWrapper, null,
        React.createElement(C.Wrapper, null,
            hasStartingTrail$
                && (React.createElement(React.Fragment, null,
                    React.createElement(C.Item, { position: 1, isCurrentPage: false, key: "startingTrail" },
                        React.createElement(C.StartingTrail, null)),
                    (items$.length > 0 || hasFinalTrail$)
                        && React.createElement(C.Separator, { key: "startingTrailSeparator" }))),
            items$,
            hasFinalTrail$
                && (React.createElement(React.Fragment, null,
                    items$.length > 0
                        && React.createElement(C.Separator, { key: "finalTrailSeparator" }),
                    React.createElement(C.Item, { key: "finalTrail", position: finalTrailItemPosition, isCurrentPage: true },
                        React.createElement(C.FinalTrail, null)))))));
};
var BreadcrumbStartComponents = {
    StartingTrail: MenuTitle,
    Separator: 'span',
    NavWrapper: 'nav',
    Wrapper: 'ol',
    Item: withoutProps('position', 'isCurrentPage')('li'),
    Title: withoutProps(['isCurrentPage'])(MenuTitle),
    FinalTrail: MenuTitle,
};
/**
 * Clean component that renders breadcrumbs.
 * @see BreadcrumbsComponents for a list of design components.
 */
var BreadcrumbsClean = designable(BreadcrumbStartComponents, 'Breadcrumbs')(BreadcrumbsClean$);
/**
 * HOC that enables rendering of starting trail for a breadcrumb based component.
 * @param Component a breadcrumb based component
 */
var withStartingTrail = addProps({
    hasStartingTrail: true,
});
/**
 * HOC that disables rendering of starting trail for a breadcrumb based component.
 * @param Component a breadcrumb based component
 */
var withoutStartingTrail = addProps({
    hasStartingTrail: false,
});
/**
 * HOC that enables rendering of final trail for a breadcrumb based component.
 * @param Component a breadcrumb based component
 */
var withFinalTrail = addProps({
    hasFinalTrail: true,
});
/**
 * HOC that disables rendering of final trail for a breadcrumb based component.
 * @param Component a breadcrumb based component
 */
var withoutFinalTrail = addProps({
    hasFinalTrail: false,
});
/**
 * HOC that adds breadcrumb props retrieved from breadcrumb store.
 */
var asBreadcrumbs = flowRight(observer, withBreadcrumbItemsFromStore, asStylableBreadcrumbs, withBreadcrumbsSD);
/**
 * Component that renders breadcrumb items retrieved from breadcrumb store.
 */
var MenuBreadcrumbs = asBreadcrumbs(BreadcrumbsClean);
export { asBreadcrumbs, BreadcrumbsClean, MenuBreadcrumbs, withStartingTrail as withBreadcrumbStartingTrail, withoutStartingTrail as withoutBreadcrumbStartingTrail, withFinalTrail as withBreadcrumbFinalTrail, withoutFinalTrail as withoutBreadcrumbFinalTrail, };
//# sourceMappingURL=Breadcrumbs.js.map