/**
 * Copyright © 2020 Johnson & Johnson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import React from 'react';
import type { HOC } from '@bodiless/fclasses';
import type { BreadcrumbItemType } from './BreadcrumbStore';
/**
 * A Hook to get the current Breadcrumb Item context values.
 */
export declare const useBreadcrumbContext: () => BreadcrumbItemType | undefined;
export declare const BreadcrumbContextProvider: React.Provider<BreadcrumbItemType | undefined>;
/**
 * Hook which can be used to determine if a menu item is part of
 * the current active breadcrumb trail.
 *
 * This hook is only accurate if
 * - The menu is inside a BreadcrumbStoreProvider.
 * - The menu item has been wrapped in asBreadcrumb
 *
 * @return true if the item is in the active trail, false otherwise.
 */
export declare const useIsActiveTrail: () => boolean;
export declare type BreadcrumbSettings = {
    linkNodeKey: string;
    titleNodeKey: string;
};
/**
 * Creates an HOC which specifies that a wrapped component is a breadcrumb. The HOC
 * will read link and title from the specified nodekeys and will push link and title
 * to the breadcrumb store. Once the wrapped component is unmounted, the corresponding link
 * and title are deleted from the breadcrumb store
 *
 * @param settings The title and link nodekeys defining where to locate the link and title nodes.
 *
 * @return An HOC which defines the wrapped component as a breadcrumb.
 */
declare const asBreadcrumb: ({ linkNodeKey, titleNodeKey, }: BreadcrumbSettings) => HOC;
/**
 * Use this HOC to wrap a menu so that it can serve a source of data for breadcrumbs
 * and menu trails. It creates a hidden version of the menu which is rendered only during
 * SSR, to ensure the breadcrumb data is pre-generated.
 *
 * You should wrap the whole menu in this HOC. You must also wrap the menu's items in
 * `asBreadcrumb`, and the all must be rendered within a BreadcrumbStoreContext
 *
 * @param Component
 * The component providing the menu data structure.
 *
 * @return
 * A version of the component which renders a hidden version of itself during SSR.
 *
 * @see asHiddenBreadcrumbSource
 * @see asBreadcrumb
 */
declare const asBreadcrumbSource: HOC;
export default asBreadcrumb;
export { asBreadcrumbSource };
//# sourceMappingURL=asBreadcrumb.d.ts.map