/**
 * Copyright © 2020 Johnson & Johnson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { createContext, useContext, useLayoutEffect, } from 'react';
import { useNode } from '@bodiless/core';
import { observer } from 'mobx-react';
import { BreadcrumbItem } from './BreadcrumbStore';
import { useBreadcrumbStore, asHiddenBreadcrumbSource } from './BreadcrumbStoreProvider';
var breadcrumbContext = createContext(undefined);
/**
 * A Hook to get the current Breadcrumb Item context values.
 */
export var useBreadcrumbContext = function () { return useContext(breadcrumbContext); };
export var BreadcrumbContextProvider = breadcrumbContext.Provider;
/**
 * Hook which can be used to determine if a menu item is part of
 * the current active breadcrumb trail.
 *
 * This hook is only accurate if
 * - The menu is inside a BreadcrumbStoreProvider.
 * - The menu item has been wrapped in asBreadcrumb
 *
 * @return true if the item is in the active trail, false otherwise.
 */
export var useIsActiveTrail = function () { var _a; return !!((_a = useBreadcrumbContext()) === null || _a === void 0 ? void 0 : _a.isActive()); };
var isSSR = function () { return !(typeof window !== 'undefined'
    && window.document
    && window.document.createElement); };
/**
 * Creates an HOC which specifies that a wrapped component is a breadcrumb. The HOC
 * will read link and title from the specified nodekeys and will push link and title
 * to the breadcrumb store. Once the wrapped component is unmounted, the corresponding link
 * and title are deleted from the breadcrumb store
 *
 * @param settings The title and link nodekeys defining where to locate the link and title nodes.
 *
 * @return An HOC which defines the wrapped component as a breadcrumb.
 */
var asBreadcrumb = function (_a) {
    var linkNodeKey = _a.linkNodeKey, titleNodeKey = _a.titleNodeKey;
    return function (Component) {
        var AsBreadcrumb = observer(function (props) {
            var current = useBreadcrumbContext();
            var store = useBreadcrumbStore();
            if (store === undefined)
                return React.createElement(Component, __assign({}, props));
            var node = useNode().node;
            var titleNode = node.child(titleNodeKey);
            var linkNode = node.child(linkNodeKey);
            // We need an id which will be the same for all breadcrumb sources which
            // render the same data.  Node path works well for this.
            var id = node.path.join('$');
            var item = new BreadcrumbItem({
                uuid: id,
                title: {
                    data: titleNode.data.text,
                    nodePath: __spreadArray(__spreadArray([], node.path, true), [titleNodeKey], false).join('$'),
                },
                link: {
                    data: linkNode.data.href,
                    nodePath: __spreadArray(__spreadArray([], node.path, true), [linkNodeKey], false).join('$'),
                },
                parent: current,
                store: store,
            });
            var isProduction = process.env.NODE_ENV === 'production';
            if (isProduction) {
                // To avoid flicker, we need to populate the store on render
                // otherwise the breadcrumbs render with no items before
                // a layout effect is executed.
                store.setItem(item);
            }
            useLayoutEffect(function () {
                store.setItem(item);
                return function () {
                    store.deleteItem(item);
                };
            }, [titleNode.data, linkNode.data]);
            return (React.createElement(BreadcrumbContextProvider, { value: item },
                React.createElement(Component, __assign({}, props))));
        });
        return AsBreadcrumb;
    };
};
/**
 * Use this HOC to wrap a menu so that it can serve a source of data for breadcrumbs
 * and menu trails. It creates a hidden version of the menu which is rendered only during
 * SSR, to ensure the breadcrumb data is pre-generated.
 *
 * You should wrap the whole menu in this HOC. You must also wrap the menu's items in
 * `asBreadcrumb`, and the all must be rendered within a BreadcrumbStoreContext
 *
 * @param Component
 * The component providing the menu data structure.
 *
 * @return
 * A version of the component which renders a hidden version of itself during SSR.
 *
 * @see asHiddenBreadcrumbSource
 * @see asBreadcrumb
 */
var asBreadcrumbSource = function (Component) {
    var SSRSource = asHiddenBreadcrumbSource(Component);
    var AsBreadcrumbSource = function (props) { return (React.createElement(React.Fragment, null,
        isSSR() && React.createElement(SSRSource, __assign({}, props)),
        React.createElement(Component, __assign({}, props)))); };
    return AsBreadcrumbSource;
};
export default asBreadcrumb;
export { asBreadcrumbSource };
//# sourceMappingURL=asBreadcrumb.js.map