/**
 * Copyright © 2020 Johnson & Johnson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { createContext, useContext, useState, } from 'react';
var BurgerMenuContext = createContext({
    isVisible: false,
    isTransitionComplete: false,
    toggle: function () { return null; },
    setIsTransitionComplete: function () { return null; },
});
/**
 * A Hook to get the current Burger Menu `isVisible` and `toggle()` context values.
 *
 * @return The current Burger Menu `isVisible` and `toggle()` context values.
 */
var useBurgerMenuContext = function () { return useContext(BurgerMenuContext); };
/**
 * @private
 * Wraps component with the BurgerMenuContext and creates two state variables
 * that are passed to the provider:
 *  - `isVisible` - Boolean to toggle Burger Menu visibility.
 *  - `toggle()` - Visibility toggle function.
 *  - `isTransitionComplete` - Boolean that determines if BM transition is completed.
 *  - `setIsTransitionComplete()` - Transition state toggle function.
 *
 */
var withBurgerMenuProvider = function (Component) { return function (props) {
    var _a = useState(false), isVisible = _a[0], toggle = _a[1];
    var _b = useState(false), isTransitionComplete = _b[0], setIsTransitionComplete = _b[1];
    return (React.createElement(BurgerMenuContext.Provider, { value: {
            isVisible: isVisible,
            isTransitionComplete: isTransitionComplete,
            toggle: toggle,
            setIsTransitionComplete: setIsTransitionComplete,
        } },
        React.createElement(Component, __assign({}, props))));
}; };
/**
 * Hook which can be used to determine if Burger Menu is visible.
 *
 * @return true if the Burger Menu is visible, false otherwise.
 */
var useIsBurgerMenuVisible = function () { return useBurgerMenuContext().isVisible; };
/**
 * Hook which returns `true` if burger menu transitions are completed.
 * Not active means Burger menu is hidden and all animations are completed.
 * This hook useful to prevent hide animations from running when component is
 * initially mounted, but has `isVisible = false`.
 *
 * @return true if the Burger Menu is hidden and all animations are completed.
 */
var useIsBurgerTransitionCompleted = function () {
    var _a = useBurgerMenuContext(), isVisible = _a.isVisible, isTransitionComplete = _a.isTransitionComplete;
    return isTransitionComplete && !isVisible;
};
/**
 * Hook which can be used to determine if Burger Menu is hidden.
 *
 * @return true if the Burger Menu is hidden, false otherwise.
 */
var useIsBurgerMenuHidden = function () { return !useBurgerMenuContext().isVisible; };
export { withBurgerMenuProvider, useBurgerMenuContext, useIsBurgerMenuVisible, useIsBurgerMenuHidden, useIsBurgerTransitionCompleted, };
//# sourceMappingURL=BurgerMenuContext.js.map