/**
 * Copyright © 2020 Johnson & Johnson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { observer } from 'mobx-react';
import { useListContext } from '@bodiless/components';
import { useEditContext } from '@bodiless/core';
import { addClasses, removeClassesIf, addClassesIf, withDesign, flowHoc, } from '@bodiless/fclasses';
import withMenuContext, { useIsMenuOpen, useMenuContext } from './withMenuContext';
import withMenuDesign from './withMenuDesign';
import { asFlex, asRelative, withColumnDirectionStyles, withStaticOnHoverStyles, } from '../token';
import { asAccessibleMenu, asAccessibleSubMenu, withAccessibleSubMenuAttr } from './asAccessibleMenu';
import { withSubmenuContext } from './withMenuItemContext';
/*
 * Utility Styles
 * ===========================================
 */
var isMenuContextActive = function () {
    var _a = useEditContext(), isActive = _a.isActive, isEdit = _a.isEdit;
    return isEdit && isActive;
};
var isMenuContextNotActive = function () {
    var _a = useEditContext(), isActive = _a.isActive, isEdit = _a.isEdit;
    return isEdit ? !isActive : true;
};
var asVerticalSubMenu = withDesign({
    Wrapper: withColumnDirectionStyles,
});
var asResponsiveSublist = withDesign({
    Wrapper: addClasses('min-w-full'),
});
var asStaticOnHover = flowHoc(withStaticOnHoverStyles, removeClassesIf(useIsMenuOpen)('hover:static'));
var asFullWidthSublist = withDesign({
    Wrapper: addClasses('w-full left-0'),
});
var useIsSubmenuExpanded = function () {
    var _a = useEditContext(), isActive = _a.isActive, isEdit = _a.isEdit;
    var activeSubmenu = useMenuContext().activeSubmenu;
    var currentItem = useListContext().currentItem;
    return (activeSubmenu === currentItem) || (isEdit && isActive);
};
var useIsHoverEnabled = function () { return !useIsMenuOpen() && useMenuContext().activeSubmenu === undefined; };
var useIsSubmenuContracted = function () {
    var _a = useEditContext(), isActive = _a.isActive, isEdit = _a.isEdit;
    var activeSubmenu = useMenuContext().activeSubmenu;
    var currentItem = useListContext().currentItem;
    var isNotActive = isEdit ? !isActive : true;
    return (activeSubmenu !== currentItem) && isNotActive;
};
var withHoverStyles = withDesign({
    OuterWrapper: flowHoc(addClassesIf(useIsHoverEnabled)('group'), observer),
    Wrapper: flowHoc(addClasses('group-hover:flex'), addClassesIf(useIsSubmenuContracted)('hidden'), observer, addClassesIf(useIsSubmenuExpanded)('flex'), observer),
});
/*
 * Base Menu Styles
 * ===========================================
 */
var withBaseMenuStyles = withDesign({
    Wrapper: flowHoc(asFlex, asRelative, withMenuContext),
    Item: asFlex,
});
/*
 * Base Sub Menu Styles
 * ===========================================
 */
var withBaseSubMenuStyles = withDesign({
    OuterWrapper: withSubmenuContext,
    Wrapper: addClasses('absolute top-full'),
    SubmenuIndicator: addClasses('flex items-center'),
});
/*
 * List Sub Menu Styles
 * ===========================================
 */
var asListSubMenu = flowHoc(withBaseSubMenuStyles, asAccessibleSubMenu, asResponsiveSublist, asVerticalSubMenu, withHoverStyles, asRelative);
/*
 * Full Width Submenu Styles
 * ===========================================
 */
var asFullWidthSubMenu = flowHoc(withBaseSubMenuStyles, asAccessibleSubMenu, asFullWidthSublist, asStaticOnHover, withHoverStyles);
/**
 * Helper which allows specifying which submenu types should have default navigation styling added.
 *
 * @param keys List of the submenu key(s) to which the default menu styles be applied to.
 *
 * @return HOC that applies default top navigation styles based on provided keys.
 */
var asTopNav = function () {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    var TopNavDesign = {
        Main: withMenuDesign('Main')(withBaseMenuStyles, asAccessibleMenu),
        List: withMenuDesign('List')(asListSubMenu),
        Cards: withMenuDesign('Cards')(asFullWidthSubMenu),
        Columns: flowHoc(withMenuDesign('Columns', 1)(asFullWidthSubMenu), withMenuDesign('Columns', 2)(withAccessibleSubMenuAttr)),
    };
    return keys.length === 0
        ? flowHoc(TopNavDesign.Main)
        : flowHoc.apply(void 0, keys.map(function (key) { return TopNavDesign[key]; }));
};
export default asTopNav;
export { isMenuContextActive, isMenuContextNotActive, useIsSubmenuExpanded, useIsSubmenuContracted, useIsHoverEnabled, };
//# sourceMappingURL=Menu.token.js.map